<?php

namespace FiloBlu\ExtAnowaveGtmEnhancedEcommerceUaTrackingEc\Helper;

/**
 * Class Data
 * @package FiloBlu\ExtAnowaveGtmEnhancedEcommerceUaTrackingEc\Helper
 */
class Data extends \Anowave\Ec\Helper\Data
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var \Magento\Framework\App\Request\Http
     */
    protected $request;

    /**
     * @var \FiloBlu\CategoryPagination\Helper\Data
     */
    protected $_helperCategoryPagination;

    /**
     * @var \Magento\Catalog\Model\CategoryFactory
     */
    protected $categoryFactory;

    /**
     * Data constructor.
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Catalog\Api\ProductRepositoryInterface $productRepository
     * @param \Magento\Customer\Model\Session $session
     * @param \Magento\Customer\Model\GroupRegistry $groupRegistry
     * @param \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderCollectionFactory
     * @param \Magento\Sales\Model\Order\Config $orderConfig
     * @param \Magento\Framework\App\Http\Context $httpContext
     * @param \Magento\Catalog\Helper\Data $catalogData
     * @param \Magento\Catalog\Model\Product\Attribute\Repository $productAttributeRepository
     * @param \Magento\Eav\Model\ResourceModel\Entity\Attribute\Option\Collection $optionCollection
     * @param \Magento\Framework\App\Request\Http $request
     * @param \FiloBlu\CategoryPagination\Helper\Data $helperCategoryPagination
     * @param \Magento\Catalog\Model\CategoryFactory $categoryFactory
     * @param array $data
     */
       public function __construct
    (
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
        \Magento\Customer\Model\Session $session,
        \Magento\Customer\Model\GroupRegistry $groupRegistry,
        \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderCollectionFactory,
        \Magento\Sales\Model\Order\Config $orderConfig,
        \Magento\Framework\App\Http\Context $httpContext,
        \Magento\Catalog\Helper\Data $catalogData,
        \Magento\Catalog\Model\Product\Attribute\Repository $productAttributeRepository,
        \Magento\Eav\Model\ResourceModel\Entity\Attribute\Option\Collection $optionCollection,
        \Magento\Framework\App\Request\Http $request,
        \FiloBlu\CategoryPagination\Helper\Data $helperCategoryPagination,
        \Magento\Catalog\Model\CategoryFactory $categoryFactory,
        array $data = []
    )
    {
		parent::__construct(
			$context, 
			$registry,
			$productRepository,
			$session,
			$groupRegistry,
			$orderCollectionFactory,
			$orderConfig,
			$httpContext,
			$catalogData,
			$productAttributeRepository,
			$optionCollection,
			$request,
			$helperCategoryPagination
		);

        $this->request = $request;
        $this->categoryFactory = $categoryFactory;
        $this->_logger = $context->getLogger();
        $this->_helperCategoryPagination = $helperCategoryPagination;
    }

    /**
     * @param \Anowave\Ec\Helper\unknown $block
     * @param \Magento\Checkout\Model\Cart $cart
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\ObjectManagerInterface $object
     * @return array
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getCheckoutProducts($block, \Magento\Checkout\Model\Cart $cart, \Magento\Framework\Registry $registry, \Magento\Framework\ObjectManagerInterface $object)
    {
        $products = [];

        foreach ($cart->getQuote()->getAllVisibleItems() as $item) {
            /**
             * Get all product categories
             */
            $categories = $item->getProduct()->getCategoryIds();

            /**
             * Load first category
             */
            $category = $this->categoryFactory->create()->load(reset($categories));

            $categoryName = '';
            if ($category && $category->getId()) {
                $categoryName = $this->getCategory($category);
            }

            $data = new \Magento\Framework\DataObject(array
            (
                'id' => $item->getSku(),
                'name' => $item->getName(),
                'price' => $item->getPriceInclTax(),
                'quantity' => (int)$item->getQty(),
                'category' => $categoryName,
                'brand' => $this->getBrand
                (
                    $item->getProduct()
                )
            ));

            if ('configurable' == $item->getProduct()->getTypeId()) {
                $variant = [];

                /**
                 * Get info buy request
                 *
                 * @var \Magento\Framework\DataObject
                 */
                $buyRequest = $item->getProduct()->getCustomOption('info_buyRequest');

                if (isset($buyRequest['value'])) {
                    $value = unserialize($buyRequest['value']);
                    $info = new \Magento\Framework\DataObject($value);
                } else {
                    $info = new \Magento\Framework\DataObject([]);
                }

                /**
                 * Construct variant
                 */
                foreach ((array)$info->getSuperAttribute() as $id => $option) {
                    /**
                     * @todo: Pull attribute data
                     */
                    $attribute = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Catalog\Model\ResourceModel\Eav\Attribute')->load($id);

                    if ($attribute->usesSource()) {
                        $name = ($this->useDefaultValues() ? $attribute->getFrontendLabel() : $attribute->getStoreLabel());
                        $text = $attribute->getSource()->getOptionText($option);

                        if ($this->useDefaultValues()) {
                            /**
                             * Get current store
                             *
                             * @var int
                             */
                            $currentStore = $attribute->getSource()->getAttribute()->getStoreId();

                            /**
                             * Change default store
                             */
                            $attribute->getSource()->getAttribute()->setStoreId(0);

                            /**
                             * Get text
                             *
                             * @var string
                             */
                            $text = $attribute->getSource()->getOptionText($option);

                            /**
                             * Restore store
                             */
                            $attribute->getSource()->getAttribute()->setStoreId($currentStore);
                        }

                        $variant[] = join(':', [$name, $text]);
                    }
                }

                $data->setId
                (
                    $item->getProduct()->getSku()
                );

                $data->setName
                (
                    $item->getProduct()->getName()
                );

                /**
                 * Load configurable
                 */
                $configurable = $this->productRepository->getById
                (
                    $item->getProductId()
                );

                /**
                 * Swap data with configurable
                 */
                $data->setId($configurable->getSku());
                $data->setName($configurable->getName());
                $data->setBrand
                (
                    $this->getBrand($configurable)
                );

                /**
                 * Push variant to data
                 *
                 * @var array
                 */
                $data->setVariant(join(chr(44), $variant));
            }

            $products[] = $data->getData();
        }

        return $products;
    }

    /**
     * @param \Magento\Catalog\Model\Category $category
     * @return string
     */
    public function getCategory(\Magento\Catalog\Model\Category $category)
    {
        if (!$category || !$category->getId()) {
            return '';
        }
        if (0 !== (int)$this->getConfig('ec/options/use_segments')) {
            return $this->getCategorySegments($category);
        }

        $useSeoCategoryNameIExist = $this->scopeConfig->getValue('mageworx_seo/seoxtemplates/use_category_seo_name', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $this->getStore()) && $category->getCategorySeoName();
        if ($useSeoCategoryNameIExist) {
            return $category->getCategorySeoName();
        }

        return $category->getName();
    }

    /**
     * Retrieve category and it's parents separated by chr(47)
     *
     * @param \Magento\Catalog\Model\Category $category
     * @return string
     */
    public function getCategorySegments(\Magento\Catalog\Model\Category $category)
    {
        $segments = array();

        foreach ($category->getParentCategories() as $parent) {
            $useSeoCategoryNameIExist = $this->scopeConfig->getValue('mageworx_seo/seoxtemplates/use_category_seo_name', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $this->getStore()) && $parent->getCategorySeoName();
            if ($useSeoCategoryNameIExist) {
                $segments[] = $parent->getCategorySeoName();
            } else {
                $segments[] = $parent->getName();
            }
        }

        if (!$segments) {
            $useSeoCategoryNameIExist = $this->scopeConfig->getValue('mageworx_seo/seoxtemplates/use_category_seo_name', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $this->getStore()) && $category->getCategorySeoName();
            if ($useSeoCategoryNameIExist) {
                $segments[] = $category->getCategorySeoName();
            } else {
                $segments[] = $category->getName();
            }
        }

        return trim(join(chr(47), $segments));
    }

    /**
     * @param \Anowave\Ec\Helper\Block $block
     * @return bool|object
     */
    public function getImpressionPushForward($block)
    {
        $response = null;
        $categoryListName = '';

        try {
            $list = $block->getLayout()->getBlock('category.products.list');

            if ($list) {
                $categoryList = $this->registry->registry('current_category');
                $categoryListName = '';

                if ($categoryList && $categoryList->getId()) {
                    $categoryListName = $this->getCategoryList($categoryList);
                }

                $response = [
                    'currencyCode' => $this->getStore()->getCurrentCurrencyCode(),
                    'ecommerce' => [
                        'actionField' => ['list' => $categoryListName],
                        'impressions' => []
                    ]
                ];

                $position = 1;
                $categories = [];

                foreach ($this->getLoadedCollection($list) as $product) {

                    $categoriesId = $product->getCategoryIds();

                    /**
                     * Load first category
                     */
                    $catId = reset($categoriesId);
                    if (!isset($categories[$catId])) {
                        $category = $this->categoryFactory->create()->load($catId);
                        $categories[$catId] = $category;
                    } else {
                        $category = $categories[$catId];
                    }
                    $categoryName = '';
                    if ($category && $category->getId()) {
                        $categoryName = $this->getCategory($category);
                    }

                    $response['ecommerce']['impressions'][] = [
                        'list' => $categoryListName,
                        'category' => $categoryName,
                        'id' => $product->getSku(),
                        'name' => $product->getName(),
                        'brand' => $this->getBrand($product),
                        'price' => $this->getPrice($product),
                        'position' => $position++
                    ];
                }

                $response['currentStore'] = $this->getStoreName();
            }

            if ($response) {
                return (object)
                [
                    'push' => json_encode($response, JSON_PRETTY_PRINT),
                    'google_tag_params' => array
                    (
                        'ecomm_pagetype' => 'category',
                        'ecomm_category' => $this->escape($categoryListName)
                    )
                ];
            } else {
                return false;
            }

        } catch (\Exception $e) {
            $this->_logger->error($e->getMessage());
        }

        return false;
    }

    /**
     * @param \Magento\Catalog\Model\Category $category
     * @return string
     */
    public function getCategoryList(\Magento\Catalog\Model\Category $category)
    {
        if (!$category || !$category->getId()) {
            return '';
        }

        $useSeoCategoryNameIExist = $this->scopeConfig->getValue('mageworx_seo/seoxtemplates/use_category_seo_name', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $this->getStore()) && $category->getCategorySeoName();
        if ($useSeoCategoryNameIExist) {
            return $category->getCategorySeoName();
        }

        return $category->getName();
    }

    /**
     * Get loaded product collection from product list block
     *
     * @param \Magento\Catalog\Block\Product\ListProduct $list
     * @return \Magento\Eav\Model\Entity\Collection\AbstractCollection
     */
    private function getLoadedCollection(\Magento\Catalog\Block\Product\ListProduct $list)
    {
        $collection = $list->getLoadedProductCollection();

        /**
         * Get toolbar
         */
        $toolbar = $list->getToolbarBlock();

        if ($toolbar) {
            $orders = $list->getAvailableOrders();

            if ($orders) {
                $toolbar->setAvailableOrders($orders);
            }

            $sort = $list->getSortBy();

            if ($sort) {
                $toolbar->setDefaultOrder($sort);
            }

            $dir = $list->getDefaultDirection();

            if ($dir) {
                $toolbar->setDefaultDirection($dir);
            }

            $modes = $list->getModes();

            if ($modes) {
                $toolbar->setModes($modes);
            }

            $limit = (int)$toolbar->getLimit();

            if ($limit) {
                $_categoryId = -1;

                // for avoid problems on search
                if ($this->registry->registry("current_category")) {
                    $_categoryId = $this->registry->registry("current_category")->getId();
                }

                $offset = 0;
                $page = (int)$this->request->getParam('p', 1);

                if ($page > 1) {

                    $listMode = $toolbar->getCurrentMode();
                    $productNumberPage1 = $this->_helperCategoryPagination->getProductNumber($listMode, 1, $_categoryId);
                    $productNumberPageOther = $this->_helperCategoryPagination->getProductNumber($listMode, 2, $_categoryId);

                    $offset = $productNumberPage1 + ($productNumberPageOther * ($page - 2));

                    if ($this->_helperCategoryPagination->getConfig('general/load_all_products_before') && !$this->request->isXmlHttpRequest()) {
                        $limit = $offset + $limit;
                        $offset = 0;
                    }
                }

                $collection->getSelect()->limit($limit, $offset);
            }

            if ($toolbar->getCurrentOrder()) {
                $collection->setOrder($toolbar->getCurrentOrder(), $toolbar->getCurrentDirection());
            }
        }

        return $collection;
    }

    /**
     * @param $block
     * @return bool|object
     */
    public function getDetailPushForward($block)
    {
        $info = $block->getLayout()->getBlock('product.info');

        if ($info) {
            $categoryList = $this->registry->registry('current_category');

            $categoryListName = '';
            if ($categoryList && $categoryList->getId()) {
                $categoryListName = $this->getCategoryList($categoryList);
            }

            /**
             * Get all product categories
             */
            $categories = $info->getProduct()->getCategoryIds();

            /**
             * Load first category
             */
            $category = $this->categoryFactory->create()->load(reset($categories));

            $categoryName = '';
            if ($category && $category->getId()) {
                $categoryName = $this->getCategory($category);
            }

            $data = [
                'currencyCode' => $this->getStore()->getCurrentCurrencyCode(),
                'ecommerce' => [
                    'detail' =>
                        [
                            'actionField' => ['list' => $categoryListName],
                            'products' => [
                                [
                                    'id' => $info->getProduct()->getSku(),
                                    'name' => $info->getProduct()->getName(),
                                    'price' => $this->getPrice($info->getProduct()),
                                    'brand' => $this->getBrand($info->getProduct()),
                                    'category' => $categoryName,
                                    'quantity' => 1
                                ]
                            ]
                        ]
                ]
            ];

            $data['currentStore'] = $this->getStoreName();

            return (object)
            [
                'push' => json_encode($data, JSON_PRETTY_PRINT),
                'fbq' => json_encode($this->getFacebookViewContentTrack($info->getProduct(), $category)),
                'google_tag_params' => [
                    'ecomm_pagetype' => 'product',
                    'ecomm_category' => $this->escape($categoryName),
                    'ecomm_prodid' => $this->escape($info->getProduct()->getSku()),
                    'ecomm_totalvalue' => $this->getPrice($info->getProduct())
                ],
                'group' => $this->getDetailGroup($info, $category)
            ];
        }

        return false;
    }

    /**
     * @param $block
     * @return false|string
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getPurchasePush($block)
    {
        $response = [];

        foreach ($this->getOrders($block) as $order) {
            $response = array
            (
                'ecommerce' => array
                (
                    'currencyCode' => $this->getStore()->getCurrentCurrencyCode(),
                    'purchase' =>
                        [
                            'actionField' =>
                                [
                                    'id' => $order->getIncrementId(),
                                    'revenue' => $order->getBaseGrandTotal(),
                                    'tax' => $order->getBaseTaxAmount(),
                                    'shipping' => $order->getBaseShippingAmount(),
                                    'coupon' => (string)$order->getCouponCode(),
                                    'affiliation' => (string)$this->getStore()->getName()
                                ],
                            'products' => []
                        ]
                ),
                'facebook' =>
                    [
                        'revenue' => $order->getBaseGrandTotal(),
                        'subtotal' => $order->getBaseSubtotal()
                    ]
            );

            if ($order->getIsVirtual()) {
                $address = $order->getBillingAddress();
            } else {
                $address = $order->getShippingAddress();
            }

            foreach ($order->getAllVisibleItems() as $item) {

                /**
                 * Get all product categories
                 */
                $categories = $item->getProduct()->getCategoryIds();

                /**
                 * Load first category
                 */
                $category = $this->categoryFactory->create()->load(reset($categories));

                $categoryName = '';
                if ($category && $category->getId()) {
                    $categoryName = $this->getCategory($category);
                }

                $data = new \Magento\Framework\DataObject(array
                (
                    'id' => $item->getSku(),
                    'name' => $item->getName(),
                    'price' => $item->getBasePrice(),
                    'quantity' => (int)$item->getQtyOrdered(),
                    'category' => $categoryName,
                    'brand' => $this->getBrand($item->getProduct())
                ));

                if ('configurable' == $item->getProduct()->getTypeId()) {
                    $variant = array();

                    /**
                     * Get info buy request
                     *
                     * @var \Magento\Framework\DataObject
                     */
                    $info = new \Magento\Framework\DataObject($item->getProductOptionByCode('info_buyRequest'));

                    /**
                     * Construct variant
                     */
                    foreach ($info->getSuperAttribute() as $id => $option) {
                        /**
                         * @todo: Pull attribute data
                         */
                        $attribute = \Magento\Framework\App\ObjectManager::getInstance()->create('Magento\Catalog\Model\ResourceModel\Eav\Attribute')->load($id);

                        if ($attribute->usesSource()) {
                            $name = ($this->useDefaultValues() ? $attribute->getFrontendLabel() : $attribute->getStoreLabel());
                            $text = $attribute->getSource()->getOptionText($option);

                            if ($this->useDefaultValues()) {
                                /**
                                 * Get current store
                                 *
                                 * @var int
                                 */
                                $currentStore = $attribute->getSource()->getAttribute()->getStoreId();

                                /**
                                 * Change default store
                                 */
                                $attribute->getSource()->getAttribute()->setStoreId(0);

                                /**
                                 * Get text
                                 *
                                 * @var string
                                 */
                                $text = $attribute->getSource()->getOptionText($option);

                                /**
                                 * Restore store
                                 */
                                $attribute->getSource()->getAttribute()->setStoreId($currentStore);
                            }

                            $variant[] = join(':', array($name, $text));
                        }
                    }

                    $data->setId
                    (
                        $item->getProduct()->getSku()
                    );

                    $data->setName
                    (
                        $item->getProduct()->getName()
                    );

                    $product = $this->productRepository->getById
                    (
                        $item->getProductId()
                    );

                    if (false) {
                        /**
                         * Get parents
                         *
                         * @var array
                         */
                        $parents = (array)\Magento\Framework\App\ObjectManager::getInstance()->create('Magento\ConfigurableProduct\Model\Product\Type\Configurable')->getParentIdsByChild
                        (
                            $product->getId()
                        );

                        $configurable = $this->productRepository->getById
                        (
                            end($parents)
                        );
                    }

                    /**
                     * Push variant to data
                     *
                     * @var array
                     */
                    $data->setVariant(join(chr(44), $variant));
                }

                $response['ecommerce']['purchase']['products'][] = $data->getData();
            }

            $response['currentStore'] = $this->getStoreName();
        }

        return json_encode($response, JSON_PRETTY_PRINT);
    }

    /**
     * @param $block
     * @return bool|object
     */
    public function getSearchPush($block)
    {
        $response = null;

        try {
            $list = $block->getLayout()->getBlock('search_result_list');

            if ($list) {
                $response = [
                    'currencyCode' => $this->getStore()->getCurrentCurrencyCode(),
                    'ecommerce' => [
                        'actionField' =>[
                            'list' => 'Search Results'
                        ],
                        'impressions' => []
                    ]
                ];

                $position = 1;

                foreach ($this->getLoadedCollection($list) as $product) {

                    $categories = $product->getCategoryIds();

                    /**
                     * Load first category
                     */
                    $category = $this->categoryFactory->create()->load
                    (
                        reset($categories)
                    );
                    $categoryName = '';
                    if ($category && $category->getId()) {
                        $categoryName = $this->getCategory($category);
                    }

                    $response['ecommerce']['impressions'][] = array
                    (
                        'list' => 'Search Results',
                        'category' => $categoryName,
                        'id' => $product->getSku(),
                        'name' => $product->getName(),
                        'brand' => $this->getBrand
                        (
                            $product
                        ),
                        'price' => $this->getPrice($product),
                        'position' => $position++
                    );
                }

                $response['currentStore'] = $this->getStoreName();
            }

            if ($response) {
                return (object)
                [
                    'push' => json_encode($response, JSON_PRETTY_PRINT),
                    'google_tag_params' => array
                    (
                        'ecomm_pagetype' => 'category',
                        'ecomm_category' => 'Search Results'
                    )
                ];
            } else {
                return false;
            }


        } catch (\Exception $e) {
            $this->_logger->error($e->getMessage());
        }

        return false;
    }

}
